import requests
import pandas


def find_near(lat, lon, *, n=10, session=None):
    """Return n results for a given latitude and longitude"""
    search_params = {'npoints': n, 'clat': lat, 'clon': lon, 
                     'Columns[]': ['Subregion', 'Notes', 'CollectionYear', 
                                   'ReservoirAge', 'ReservoirErr', 'C14age', 
                                   'C14err', 'LabID', 'Delta13C', 'nextime', 
                                   'Genus', 'Species', 'Feeding', 'Name']}
    resp = _query_near(session=session, **search_params)
    df = _response_to_dataframe(resp)
    df_clean = _clean_dataframe(df)
    return df_clean


def _query_near(*, session=None, **kwargs):
    """Query marine database with given query string values and keys"""
    url_endpoint = 'http://calib.org/marine/index.html'
    if session is not None:
        resp = session.get(url_endpoint, params=kwargs)
    else:
        with requests.Session() as s:
            # Need to get the index page before query. Otherwise get bad query response that seems legit.
            s.get('http://calib.org/marine/index.html')
            resp = s.get(url_endpoint, params=kwargs)
    return resp


def _response_to_dataframe(x):
    """Convert tables in HTML response, x, to list of pandas dataframes"""
    df = pandas.read_html(x.text, header=0)
    return df


def _clean_dataframe(df):
    """Clean response list of pandas dataframes to query results table"""
    cleaned = df[3]
    del cleaned['Unnamed: 0']
    del cleaned['Unnamed: 23']
    return cleaned



import unittest
import carbonferret as cf
from os import path


class DummyResponse:

    def __init__(self, payload):
        self.text = payload


class DummySession:

    def __init__(self, testhtml):
        self.response = DummyResponse(testhtml)

    def get(self, url_endpoint, params):
        return self.response


class TestCarbonferret(unittest.TestCase):

    def setUp(self):
        # testresponse.html was taken from calib.org/marine on 2017-05-02
        here = path.abspath(path.dirname(__file__))
        testhtml_path = path.join(here, 'testresponse.html')
        with open(testhtml_path) as fl:
            testhtml_string = fl.read()
        self.testhtml_string = testhtml_string
        self.dummysession = DummySession(testhtml_string)

    # @unittest.skip('Test not written')
    def test_find_near(self):
        victim = cf.find_near(lat=0, lon=0, n=2, session=self.dummysession)
        # Just testing a couple key values, not everything.
        goals = {'MapNo': [1221, 1222],
                'Lon': [-5.7416, -5.7416],
                'Lat': [-15.9409, -15.9409],
                'DeltaR': [361, 294]}
        for k, v in goals.items():
            self.assertTrue(all(victim[k].values == v))


if __name__ == '__main__':
    unittest.main()



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

setup(
    name='carbonferret',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.0.1',

    description='Use Python to quickly query carbon reservoir records at http://calib.org/marine/',

    # The project's main homepage.
    url='https://github.com/brews/carbonferret',

    # Author details
    author='S. Brewster Malevich',
    author_email='malevich@email.arizona.edu',

    # Choose your license
    license='GPLv3',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'Topic :: Software Development',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3',
    ],

    # What does your project relate to?
    keywords='marine carbonreservoir c14',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(exclude=['contrib', 'docs']),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=['requests', 'beautifulsoup4', 'html5lib', 'lxml', 'pandas'],

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
        'dev': ['check-manifest']
    }
)
